#
# Copyright (c) 2008-2017 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
from persistent import Persistent

# import interfaces
from pyams_content.component.illustration.interfaces import IIllustration
from pyams_content.component.paragraph.interfaces.verbatim import IVerbatimParagraph
from pyams_content.features.renderer.interfaces import IContentRenderer
from pyams_default_theme.component.paragraph.interfaces.verbatim import IVerbatimParagraphRendererSettings, \
    ILateralVerbatimParagraphRendererSettings, IDefaultVerbatimParagraphRendererSettings
from pyams_i18n.interfaces import II18n
from pyams_skin.layer import IPyAMSLayer

# import packages
from pyams_content.features.renderer.skin import BaseContentRenderer
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config, get_annotation_adapter
from zope.interface import implementer
from zope.location import locate, Location
from zope.schema.fieldproperty import FieldProperty

from pyams_default_theme import _


#
# Verbatim paragraph default renderer settings
#

VERBATIM_PARAGRAPH_RENDERER_SETTINGS_KEY = 'pyams_content.verbatim.renderer:default'


@implementer(IVerbatimParagraphRendererSettings)
class BaseVerbatimParagraphRendererSettings(Persistent, Location):
    """Base frame text paragraph renderer settings"""

    display_illustration = FieldProperty(IVerbatimParagraphRendererSettings['display_illustration'])

    def can_display_illustration(self):
        if not self.display_illustration:
            return False
        frame = IVerbatimParagraph(self.__parent__)
        illustration = IIllustration(frame, None)
        return (illustration is not None) and bool(II18n(illustration).query_attribute('data'))


@implementer(IDefaultVerbatimParagraphRendererSettings)
class DefaultVerbatimParagraphRendererSettings(BaseVerbatimParagraphRendererSettings):
    """Verbatim paragraph lateral renderer settings"""


@adapter_config(context=IVerbatimParagraph, provides=IDefaultVerbatimParagraphRendererSettings)
def default_verbatim_paragraph_renderer_settings_factory(context):
    """Frame paragraph default renderer settings factory"""
    return get_annotation_adapter(context, VERBATIM_PARAGRAPH_RENDERER_SETTINGS_KEY,
                                  DefaultVerbatimParagraphRendererSettings)


@implementer(ILateralVerbatimParagraphRendererSettings)
class LateralVerbatimParagraphRendererSettings(BaseVerbatimParagraphRendererSettings):
    """Verbatim paragraph lateral renderer settings"""

    relative_width = FieldProperty(ILateralVerbatimParagraphRendererSettings['relative_width'])


LATERAL_VERBATIM_PARAGRAPH_RENDERER_SETTINGS_KEY = 'pyams_content.verbatim.renderer:lateral'


@adapter_config(context=IVerbatimParagraph, provides=ILateralVerbatimParagraphRendererSettings)
def lateral_verbatim_paragraph_renderer_settings_factory(context):
    """Frame text paragraph lateral renderer settings factory"""
    return get_annotation_adapter(context, LATERAL_VERBATIM_PARAGRAPH_RENDERER_SETTINGS_KEY,
                                  LateralVerbatimParagraphRendererSettings)


#
# Verbatim paragraph default renderer
#

class BaseVerbatimParagraphRenderer(BaseContentRenderer):
    """Base frame paragraph renderer"""

    context_attrs = ('author', )
    i18n_context_attrs = ('title', 'quote', 'charge')
    illustration_renderer = None

    def update(self):
        super(BaseVerbatimParagraphRenderer, self).update()
        if self.settings.can_display_illustration():
            illustration = IIllustration(self.context, None)
            renderer = illustration.get_renderer(self.request)
            if renderer is not None:
                renderer.update()
                self.illustration_renderer = renderer


@adapter_config(name='default', context=(IVerbatimParagraph, IPyAMSLayer), provides=IContentRenderer)
@template_config(template='templates/verbatim-default.pt', layer=IPyAMSLayer)
class DefaultVerbatimParagraphRenderer(BaseVerbatimParagraphRenderer):
    """Verbatim paragraph default renderer"""

    label = _("Default verbatim renderer")
    weight = 1

    settings_interface = IDefaultVerbatimParagraphRendererSettings


@adapter_config(name='left', context=(IVerbatimParagraph, IPyAMSLayer), provides=IContentRenderer)
@template_config(template='templates/verbatim-left.pt', layer=IPyAMSLayer)
class LeftVerbatimParagraphRenderer(BaseVerbatimParagraphRenderer):
    """Verbatim paragraph renderer displayed on the left"""

    label = _("Small frame on the left")
    weight = 2

    settings_interface = ILateralVerbatimParagraphRendererSettings


@adapter_config(name='right', context=(IVerbatimParagraph, IPyAMSLayer), provides=IContentRenderer)
@template_config(template='templates/verbatim-right.pt', layer=IPyAMSLayer)
class RightVerbatimParagraphRenderer(BaseVerbatimParagraphRenderer):
    """Verbatim paragraph renderer displayed on the right"""

    label = _("Small frame on the right")
    weight = 3

    settings_interface = ILateralVerbatimParagraphRendererSettings
